/*

  Copyright 2010, 2011 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.app.stage.example;

import java.io.IOException;

import cf.common.Param;
import cf.sim.SimEngine;
import cf.sim.SimNode;
import cf.sim.stage.JavaStageCtrlCyberFramework;
import cf.sim.stage.ModelInterface;
import cf.sim.stage.devices.SimCameraStage;
import cf.sim.stage.devices.SimLaserStage;
import cf.sim.stage.devices.SimPositionStage;
import cf.sim.stage.devices.SimPowerpackStage;

class MyStageRobot extends JavaStageCtrlCyberFramework
{
  public void init() 
  {
    int node_idx = GetNodeIndex();

    // making nodes
    SimNode           robot = new SimNode(this, world(), "RobotNode_"+node_idx);
    SimNode           position = new SimNode(this, world(), robot, "PositionNode_"+node_idx);
    SimNode           camera = new SimNode(this, world(), robot,"CameraNode_" + node_idx);
    SimNode           powerpack = new SimNode(this, world(), robot, "PowerpackNode_"+node_idx);
    SimNode           laser = new SimNode(this, world(), robot, "LaserNode_"+node_idx);

    //making device
    SimPositionStage  position_dev = new SimPositionStage(this, world(), robot, "PositionDev_"+node_idx);
    SimCameraStage    camera_dev= new SimCameraStage(this, world(), robot, "CameraDev_"+node_idx);
    SimPowerpackStage powerpack_dev = new SimPowerpackStage(this, world(), robot, "PowerpackDev_"+node_idx);
    SimLaserStage     laser_dev = new SimLaserStage(this, world(), robot, "LaserDev_"+node_idx);

    // making apps
    RobotApp ra = new RobotApp(robot);
    PositionApp pa = new PositionApp(position, node_idx, position_dev);
    CameraApp ca = new CameraApp(camera, camera_dev);
    PowerpackApp ppa = new PowerpackApp(powerpack, powerpack_dev);
    LaserApp la = new LaserApp(laser, laser_dev);

    // set wifi nodes
    setWifiNode(robot); 

    ModelInterface.SetPose( node_idx*3-50 , 0+50, GetRobotP() );

    /*
       symemetric SimLink will be generated automatically btw node in Parent <-> Child  relationship 
     */

    //    SimLink l ;
    //    l = world().addSymmetricLink(camera, position, LinkType.Persistent);
    //    world().handleLinkChange(true, l);
    //    l = world().addSymmetricLink(powerpack, camera, LinkType.Persistent);
    //    world().handleLinkChange(true, l);
    //    l = world().addSymmetricLink(position, laser, LinkType.Persistent);
    //    world().handleLinkChange(true, l);
  }
}

class SensorMain
{
  public static void main (String args[]) throws IOException
  { 
    //String currentDir = System.getProperty("user.dir");

	if(Param.RealMode) 
	{
		System.out.println("Param.RealMode should be false");
		System.exit(0);
	}

	if(!Param.UseSimNetwork) 
	{
		System.out.println("Param.UseSimNetwork should be true");
		System.exit(0);
	}

	if(Param.UseSimBroadcast) 
	{
		System.out.println("Param.UseSimBroadcast should be false");
		System.exit(0);
	}
		
    boolean gui = true;
    //boolean gui = false;

    SimEngine.stage_world_gen(5, "test.world", 100, 30); // ( node_count, world_file_name , link_success_rate, end_time ) 

    SimEngine.fork_stage("cf/app/stage/example/MyStageRobot", "$CF/bin", "test.world", gui); // ( stage_controller_class_name, class_path, world_file_name, gui_on/off )
  }
}


