/*

  Copyright 2010, 2011 SRI International and Seoul National University

  Licensed under the Apache License, Version 2.0 (the "License"); you may
  not use this file except in compliance with the License. You may obtain
  a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

  The development was supported by National Science Foundation Grant 0932397:
  A Logical Framework for Self-Optimizing Networked Cyber-Physical Systems

*/

package cf.sim.stage.devices;

import cf.api.CyberApplication;
import cf.api.devices.Position;
import cf.api.devices.PositionApplication;
import cf.common.Event;
import cf.common.Model;
import cf.sim.SimWorld;
import cf.sim.stage.JavaStageCtrl;
import cf.sim.stage.ModelInterface;
import cf.sim.stage.Pose;
import cf.sim.stage.PositionInterface;

public class SimPositionStage extends Model implements Position
{    

  public JavaStageCtrl ctrl;
  private double posx, posy;

  public SimPositionStage(JavaStageCtrl ctrl, SimWorld world)
  {
    this(ctrl, world,null,null);
  }

  public SimPositionStage(JavaStageCtrl ctrl, SimWorld world, Model parent)
  {
    this(ctrl, world,parent,null);
  }

  public SimPositionStage(JavaStageCtrl ctrl, SimWorld world, Model parent, String name)
  {
    super(world,parent,name);
    this.ctrl = ctrl;
    this.posx = 0;
    this.posy = 0;
  }

  public final double posx() 
  { 
    Pose p = ModelInterface._GetPose(ctrl.GetRobotP());
    return p.x; 
  }
  public final double posy() 
  { 
    Pose p = ModelInterface._GetPose(ctrl.GetRobotP());
    return p.y; 
  }

  public final void setPos(double posx, double posy)
  {
    ModelInterface._SetPose(new Pose(posx, posy,0.0, 0.0), ctrl.GetRobotP() );
    this.posx = posx;
    this.posy = posy;
  }

  public final void move(double newx, double newy) 	
  { 
    PositionInterface._GoTo(newx, newy, 0, ctrl.GetRobotP());
  }

  public final void SetSpeed(double vel_x, double vel_y, double vel_a)
  {
    PositionInterface._SetSpeed(vel_x, vel_y, vel_a, ctrl.GetRobotP());
  }

  public final void SetTurnSpeed(double vel_a)
  {
    PositionInterface._SetTurnSpeed(vel_a, ctrl.GetRobotP());
  }

  public final void stop() 
  {
    PositionInterface._Stop(ctrl.GetRobotP());
  }

  public String logRepLong()
  {
    return super.logRepLong() + " Position: " + "(" + posx + "," + posy + ")";
  }

  public void addApplication(PositionApplication app)
  {
    addApplication((CyberApplication)app);
  }

  public void initialize()
  {
    getNode().postEvent(new InformPositionEvent(1.0));
  }

  class InformPositionEvent extends Event
  {
    InformPositionEvent(double delay)
    {
      super(delay);
    }
    public void handle()
    {
      //log("position info update");
      for(CyberApplication app : allApplications())
      {
        if(app instanceof PositionApplication) ((PositionApplication)app).handlePositionUpdate();
      }
      getNode().postEvent(new InformPositionEvent(1.0));
    }
  }

}
